--
-- Made for MF 135
--
-- Made by:
-- Martin Schmidt
-- 25/01/10
--
-- Copyright (C) 

SchmidtGearbox = {};

function SchmidtGearbox.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function SchmidtGearbox:load(xmlFile)
	


	self.gasHandle = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.gasHandle#index"));
	
	self.normalGear = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.normalGear#index"));
	self.secondaryGear = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.secondaryGear#index"));
	self.clutchPedal = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.clutchPedal#index"));
	self.BrakePedal = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.brakePedal#index"));
	self.gasPedal = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.gasPedal#index"));
	
	
	
	self.NormalGearRatio = 5;
	
	
	self.CurrentMainGear = 0;
	self.CurrentDifferentialGear = 1;
	
	self.FirstDifferentialGear = 0.25;
	self.SecondDifferentialGear = 1;
	
	self.RevGearRatio = 0.5;
	self.FirstGearRatio = 0.35;
	self.SecondGearRatio = 0.5;
	self.ThirdGearRatio = 0.7;
	self.FourthGearRatio = 1.05;
	
	
	
	self.GearboxHudX = 0.6;
    self.GearboxHudY = 1-1.14;
    self.GearboxOverlay = Overlay:new("GearboxOverlay", Utils.getFilename("Gearbox.png", self.baseDirectory), self.GearboxHudX, self.GearboxHudY, 0.256, 0.316);
	
	self.ClutchDown = false;
	

	self.showHud = false;
end;

function SchmidtGearbox:keyEvent(unicode, sym, modifier, isDown)
	if isDown and sym == Input.KEY_pageup then
		if self.CurrentMainGear < 4 then
			self.CurrentMainGear = self.CurrentMainGear + 1;
		end;
	end;	
	if isDown and sym == Input.KEY_pagedown then
		if self.CurrentMainGear > -1 then
			self.CurrentMainGear = self.CurrentMainGear - 1;
		end;
	end;
	
	if isDown and sym == Input.KEY_home then
		if 	self.CurrentDifferentialGear < 2 then
				self.CurrentDifferentialGear = 	self.CurrentDifferentialGear + 1;
		end;
	end;	
	if isDown and sym == Input.KEY_end then
		if 	self.CurrentDifferentialGear > 0 then
				self.CurrentDifferentialGear = 	self.CurrentDifferentialGear - 1;
		end;
	end;
	if isDown and sym == Input.KEY_0 then
		self.showHud = not self.showHud;
	end;
end;


function SchmidtGearbox:update(dt)
	
	
	if self.isMotorStarted and self.isEntered then
		
		
		
		local CurrentGearRatio = 0;
		local CurrentDifferentialRatio = 1;
		local rpmRange = 20;
		
		--SecondaryGearbox--
		if 	self.CurrentDifferentialGear == 0 then
			CurrentDifferentialRatio = self.FirstDifferentialGear;
		elseif 	self.CurrentDifferentialGear == 2 then
			CurrentDifferentialRatio = self.SecondDifferentialGear;
		end;
				
		--Gearbox--
		if self.CurrentMainGear == -1 then
			CurrentGearRatio = self.NormalGearRatio/self.RevGearRatio;
			rpmRange = 300;
		elseif self.CurrentMainGear == 1 then
			CurrentGearRatio = self.NormalGearRatio/self.FirstGearRatio;
			rpmRange = 200;
		elseif self.CurrentMainGear == 2 then
			CurrentGearRatio = self.NormalGearRatio/self.SecondGearRatio;
			rpmRange = 250;
		elseif self.CurrentMainGear == 3 then
			CurrentGearRatio = self.NormalGearRatio/self.ThirdGearRatio;
			rpmRange = 250;
		elseif self.CurrentMainGear == 4 then
			CurrentGearRatio = self.NormalGearRatio/self.FourthGearRatio;
			rpmRange = 0;
		end;
		CurrentGearRatio = CurrentGearRatio/CurrentDifferentialRatio;
		self.motor.backwardGearRatio = CurrentGearRatio
		self.motor.forwardGearRatios = {CurrentGearRatio/2,CurrentGearRatio/1.5,CurrentGearRatio};
		self.motor.rpmFadeOutRange = rpmRange;	
	
	
		local acceleration = 0;
		
		--Accelerator--
        if g_currentMission.allowSteerableMoving then
			if Input.isKeyPressed(Input.KEY_w) or self.motor.speedLevel ~= 0 then
				if self.CurrentDifferentialGear == 0 then
					if self.CurrentMainGear == -1 then
						acceleration = -0.02;
					elseif self.CurrentMainGear == 1 then
						acceleration = 0.02;
					elseif self.CurrentMainGear == 2 then
						acceleration = 0.025;
					elseif self.CurrentMainGear == 3 then
						acceleration = 0.035;
					elseif self.CurrentMainGear == 4 then
						acceleration = 0.04;
					end;
				elseif self.CurrentDifferentialGear == 2 then
					if self.CurrentMainGear == -1 then
						if self.motor.lastMotorRpm < 1000 then
							acceleration = -0.05;
						else
							acceleration = -0.10;
						end;
					elseif self.CurrentMainGear == 1 then
						if self.motor.lastMotorRpm < 1000 then
							acceleration = 0.05;
						else
							acceleration = 0.10;
						end;
					elseif self.CurrentMainGear == 2 then
						if self.motor.lastMotorRpm < 1000 then
							acceleration = 0.07;
						else
							acceleration = 0.15;
						end;
					elseif self.CurrentMainGear == 3 then
						acceleration = self.motor.lastMotorRpm/2300/3+0.01;
					elseif self.CurrentMainGear == 4 then
						acceleration = self.motor.lastMotorRpm/2300/1.45-0.07;
					
					end;
				end;
			end;
			
			if Input.isKeyPressed(Input.KEY_w) then
				self.motor:setSpeedLevel(0,true)
			end;
			
			
		end;
		
		--Brake--
		if Input.isKeyPressed(Input.KEY_s) then
			self.motor.brakeForce = 18;
			acceleration = 0;
		else
			self.motor.brakeForce = 0.0001;
		end;
		
		--Acceleration update--
		if self.firstTimeRun then
			if self.ClutchDown then
				acceleration = acceleration*0.35;
			end;
			WheelsUtil.updateWheelsPhysics(self, dt, self.lastSpeed, acceleration, false, self.requiredDriveMode)
		end;
		
		
		
		
		
		
		
		-- GRAFICAL MOVEMENT --
		local x, y, z = 0, 0, 0;
		
		
		-- Gearbox --
		x, y, z = getRotation(self.normalGear);
		
		-- Define Gear Positions.
		local xs = {1,0,-1,1,1,-1};
		local zs = {-1,0,-1,1,0,1};
		
		--Increment so lowest posible gear = 1.
		local gearIndex = self.CurrentMainGear+2;
		local multiplier = 0.3;
		local newX = round(xs[gearIndex]*multiplier, 2);
		local newZ = round(zs[gearIndex]*multiplier, 2);
		
		x = round(x, 2);
		y = round(y, 2);
		z = round(z, 2);
		if z == newZ and x == newX then
			self.ClutchDown = false;
		else
			self.ClutchDown = true;
			if newZ ~= z and x~=0 then
				if x > 0 then
					setRotation(self.normalGear, x-0.03, 0, z);
				elseif x < 0 then
					setRotation(self.normalGear, x+0.03, 0, z);
				end;
			elseif newZ ~= z then
				if z > newZ then
					setRotation(self.normalGear, x, 0, z-0.03);
				elseif z < newZ then
					setRotation(self.normalGear, x, 0, z+0.03);
				end;
			elseif z == newZ then
				if x > newX then
					setRotation(self.normalGear, x-0.03, 0, z);
				elseif x < newX then
					setRotation(self.normalGear, x+0.03, 0, z);
				end;
			end;
		end;
		-- /Gearbox --
		
		-- Secondary Gearbox --
		x, y, z = getRotation(self.secondaryGear);
		if 	self.CurrentDifferentialGear == 0 then
			if x < 0.4 then
				self.ClutchDown = true;
				setRotation(self.secondaryGear, x+0.01, 0, 0);
			else
				self.ClutchDown = self.ClutchDown or false;
			end;
		elseif 	self.CurrentDifferentialGear == 2 then
			if x > -0.3 then
				self.ClutchDown = true;
				setRotation(self.secondaryGear, x-0.01, 0, 0);
			else
				self.ClutchDown = self.ClutchDown or false;
			end;
		elseif self.CurrentDifferentialGear == 1 then
			if x < -0.01 then
				self.ClutchDown = true;
				setRotation(self.secondaryGear, x+0.01, 0, 0);	
			elseif x > 0.01 then
				self.ClutchDown = true;
				setRotation(self.secondaryGear, x-0.01, 0, 0);
			else
				self.ClutchDown = self.ClutchDown or false;
			end;
		end;
		-- /Secondary Gearbox --
		
		-- Accelerator --
		x, y, z = getRotation(self.gasPedal);
		if Input.isKeyPressed(Input.KEY_w) then
			if x < 0.4 then
				setRotation(self.gasPedal, x+0.01, 0, 0);	
			end;
		else
			if x > 0 then
				setRotation(self.gasPedal, x-0.01, 0, 0);	
			end;
		end;
		-- /Accelerator --
		
		
		-- AcceleratorHandle --
		x, y, z = getRotation(self.gasHandle);
		
		--setRotation(self.gasHandle, 0, (self.motor.lastMotorRpm-500)/1000, 0);	
		
		-- /AcceleratorHandle --
		
		
		
			
		-- Brake --
		x, y, z = getRotation(self.BrakePedal);
		if Input.isKeyPressed(Input.KEY_s) then
			if x > -0.14 then
				setRotation(self.BrakePedal, x-0.003, 0, 0);	
			end;
		else
			if x < 0 then
				setRotation(self.BrakePedal, x+0.003, 0, 0);	
			end;
		end;
		-- /Brake --
		
		-- Clutch --
		x, y, z = getRotation(self.clutchPedal);
		if self.ClutchDown or self.motor.lastMotorRpm <= 500 then
			if x > -0.14 then
				setRotation(self.clutchPedal, x-0.005, 0, 0);	
			end;
		else
			if x < 0 then
				setRotation(self.clutchPedal, x+0.003, 0, 0);	
			end;
		end;
		-- /Clutch --
	
	end;
end;

function SchmidtGearbox:draw()
	
		
	renderText(0.8335, 0.15, 0.02,  "Info: Press 0");
	if self.showHud then
		renderText(0.8335, 0.17, 0.02,  "RPM: " .. string.format(self.motor.lastMotorRpm) .. "");
		
		renderText(0.8335, 0.25, 0.02,  "Start Engine: 5");
		renderText(0.8335, 0.23, 0.02,  "Lift/Lower hydralics: O/L");
		renderText(0.8335, 0.21, 0.02,  "Gear: Page-up/down");
		renderText(0.8335, 0.19, 0.02,  "SecGear: Home/End");
	
	
	
	
	
	
	
		self.GearboxOverlay:render();
		
		if self.CurrentMainGear == -1 then
			setTextColor(0,1,0,1);
			renderText(0.62, 0.115, 0.035, "R");
		else
			setTextColor(1,1,1,1);
			renderText(0.62, 0.115, 0.035, "R");
		end;
		if self.CurrentMainGear == 0  then
			setTextColor(1,0,0,1);
			renderText(0.6525, 0.075, 0.035, "N");
		else
			setTextColor(1,1,1,1);
			renderText(0.6525, 0.075, 0.035, "N");
		end;
		if self.CurrentMainGear == 1 then
			setTextColor(0,1,0,1);
			renderText(0.62, 0.04, 0.035, "1");
		else
			setTextColor(1,1,1,1);
			renderText(0.62, 0.04, 0.035, "1");
		end;
		if self.CurrentMainGear == 4  then
			setTextColor(0,1,0,1);
			renderText(0.685, 0.04, 0.035, "4");
		else
			setTextColor(1,1,1,1);
			renderText(0.685, 0.04, 0.035, "4");
		end;
		if self.CurrentMainGear == 2 then
			setTextColor(0,1,0,1);
			renderText(0.685, 0.115, 0.035, "2");
		else
			setTextColor(1,1,1,1);
			renderText(0.685, 0.115, 0.035, "2");
		end;
		if self.CurrentMainGear == 3 then
			setTextColor(0,1,0,1);
			renderText(0.6525, 0.115, 0.035, "3");
		else
			setTextColor(1,1,1,1);
			renderText(0.6525, 0.115, 0.035, "3");
		end;
		
		
		if self.CurrentDifferentialGear == 1  then
			setTextColor(1,0,0,1);
			renderText(0.7725, 0.075, 0.035, "N");
		else
			setTextColor(1,1,1,1);
			renderText(0.7725, 0.075, 0.035, "N");
		end;
		if self.CurrentDifferentialGear == 0 then
			setTextColor(0,1,0,1);
			renderText(0.7725, 0.115, 0.035, "L");
		else
			setTextColor(1,1,1,1);
			renderText(0.7725, 0.115, 0.035, "L");
		end;
		if self.CurrentDifferentialGear == 2 then
			setTextColor(0,1,0,1);
			renderText(0.7725, 0.04, 0.035, "H");
		else
			setTextColor(1,1,1,1);
			renderText(0.7725, 0.04, 0.035, "H");
		end;
	end;
	
end;


function round(num, idp)
  if idp and idp>0 then
    local mult = 10^idp
    return math.floor(num * mult + 0.5) / mult
  end
  return math.floor(num + 0.5)
end


function SchmidtGearbox:onEnter()
end;
function SchmidtGearbox:onLeave()
end;
function SchmidtGearbox:delete()
end;
function SchmidtGearbox:mouseEvent(posX, posY, isDown, isUp, button)
end;